@@ -44,17 +44,28 @@ def toggle_has_read(f):
 
     @wraps(f)
     def decorated_function(*args, **kwargs):
-        assert "site" in kwargs  # only use on site-specific views
+        assert "site" in kwargs
+        assert "title_id" in kwargs
         has_read_chapter_id = request.args.get("has_read")
         unread_chapter_id = request.args.get("unread")
         assert not (has_read_chapter_id and unread_chapter_id)  # can't do both
 
         if session.get("user"):
             if has_read_chapter_id:
-                read(session["user"]["id"], kwargs["site"], has_read_chapter_id)
+                read(
+                    session["user"]["id"],
+                    kwargs["site"],
+                    kwargs["title_id"],
+                    has_read_chapter_id,
+                )
                 return redirect(request.url[: request.url.rfind("?")])
             elif unread_chapter_id:
-                unread(session["user"]["id"], kwargs["site"], unread_chapter_id)
+                unread(
+                    session["user"]["id"],
+                    kwargs["site"],
+                    kwargs["title_id"],
+                    unread_chapter_id,
+                )
                 return redirect(request.url[: request.url.rfind("?")])
         return f(*args, **kwargs)
 
