@@ -13,9 +13,6 @@ from flask import (
     url_for,
 )
 
-from mangoapi import get_chapter, get_title, search_title
-
-from . import mangadex
 from .conf import config
 from .decorators import ensure_session_version, require_login, toggle_has_read
 from .persistence import (
@@ -30,6 +27,14 @@ from .persistence import (
     unfollow,
     verify_username_password,
 )
+from .source_sites import (
+    get_chapter,
+    get_title,
+    search_title_all_sites,
+    title_cover,
+    title_source_url,
+    title_thumbnail,
+)
 
 config.load()
 
@@ -47,11 +52,13 @@ def home_view():
     return render_template("home.html")
 
 
-@app.route("/me", methods=["GET"])
+@app.route("/following", methods=["GET"])
 @ensure_session_version
 @require_login
 def follows_view():
     titles = get_followed_titles(session["user"]["id"])
+    for title in titles:
+        title["thumbnail"] = title_thumbnail(title["site"], title["id"])
     return render_template("follows.html", titles=titles)
 
 
@@ -162,7 +169,7 @@ def auth_view():
     return render_template("auth.html")
 
 
-@app.route("/title/<site>/<title_id>")
+@app.route("/m/<site>/<title_id>")
 @ensure_session_version
 @toggle_has_read
 def title_view(site, title_id):
@@ -171,33 +178,38 @@ def title_view(site, title_id):
     title = load_title(site, title_id, user_id=user_id)
     if not title:
         print("Getting title", title_id)
-        title = get_title(title_id)
+        title = get_title(site, title_id)
         print("Saving title", title_id, "to db")
         save_title(title)
     else:
         print("Loading title", title_id, "from db")
     title["site"] = site
+    title["cover"] = title_cover(site, title_id, title["cover_ext"])
+    title["source_url"] = title_source_url(site, title_id)
     return render_template("title.html", **title)
 
 
-@app.route("/chapter/<site>/<chapter_id>")
+@app.route("/m/<site>/<title_id>/<chapter_id>")
 @ensure_session_version
 @toggle_has_read
-def chapter_view(site, chapter_id):
-    chapter = load_chapter(site, chapter_id)
+def chapter_view(site, title_id, chapter_id):
+    chapter = load_chapter(site, title_id, chapter_id)
     if not chapter:
         print("Getting chapter", chapter_id)
-        chapter = get_chapter(chapter_id)
+        chapter = get_chapter(site, title_id, chapter_id)
+        chapter["site"] = site
         save_chapter(chapter)
     else:
         print("Loading chapter", chapter_id, "from db")
 
-    chapter["pages"] = [
-        url_for("proxy_view", b64_url=_encode_proxy_url(p)) for p in chapter["pages"]
-    ]
+    if site in ("mangadex", "mangasee"):
+        chapter["pages"] = [
+            url_for("proxy_view", b64_url=_encode_proxy_url(p))
+            for p in chapter["pages"]
+        ]
 
     # YIIIIKES
-    title = load_title(site, chapter["title_id"])
+    title = load_title(site, title_id)
     prev_chapter, next_chapter = get_prev_next_chapters(title, chapter)
     chapter["prev_chapter"] = prev_chapter
     chapter["next_chapter"] = next_chapter
@@ -210,11 +222,10 @@ def chapter_view(site, chapter_id):
 @ensure_session_version
 def search_view():
     query = request.args.get("q", "").strip()
-    titles = []
+    results = {}
     if query:
-        cookies = mangadex.get_cookies()
-        titles = search_title(cookies, query)
-    return render_template("search.html", titles=titles, query=query)
+        results = search_title_all_sites(query)
+    return render_template("search.html", results=results, query=query)
 
 
 @app.route("/proxy/<b64_url>")
@@ -222,6 +233,7 @@ def proxy_view(b64_url):
     """Fine I'll do it"""
     url = _decode_proxy_url(b64_url)
     if not _is_manga_img_url(url):
+        print("Invalid img url:", url)
         return "Nope", 400
     md_resp = requests.get(url)
     resp = make_response(md_resp.content, md_resp.status_code)
@@ -238,6 +250,9 @@ def _decode_proxy_url(b64_url):
 
 
 def _is_manga_img_url(
-    url, pattern=re.compile(r"^https://(\w+\.)?mangadex\.org/data/.+$")
+    url,
+    pattern=re.compile(
+        r"^https://([\w_-]+\.)?(mangadex\.org/data|mangabeast\d{0,4}.com/manga)/"
+    ),
 ):
     return pattern.match(url)
