@@ -37,7 +37,7 @@ def save_title(title):
         {
             "id": title["id"],
             "name": title["name"],
-            "site": "mangadex",
+            "site": title["site"],
             "cover_ext": title["cover_ext"],
             "chapters": json.dumps(title["chapters"]),
             "alt_names": json.dumps(title["alt_names"]),
@@ -121,7 +121,7 @@ def save_chapter(chapter):
         {
             "id": chapter["id"],
             "title_id": chapter["title_id"],
-            "site": "mangadex",
+            "site": chapter["site"],
             "num_major": chapter.get("num_major"),
             "num_minor": chapter.get("num_minor"),
             "name": chapter["name"],
@@ -132,14 +132,14 @@ def save_chapter(chapter):
     )
 
 
-def load_chapter(site, chapter_id):
+def load_chapter(site, title_id, chapter_id):
     result = run_sql(
         """
         SELECT id, title_id, num_major, num_minor, name, pages, groups, is_webtoon
         FROM chapter
-        WHERE id = ? AND site=?;
+        WHERE site=? AND title_id=? AND id=?;
         """,
-        (chapter_id, site),
+        (site, title_id, chapter_id),
     )
     if not result:
         return None
@@ -261,20 +261,21 @@ def get_followed_titles(user_id):
     return sorted(titles, key=lambda t: len(t["chapters"]), reverse=True)
 
 
-def read(user_id, site, chapter_id):
+def read(user_id, site, title_id, chapter_id):
     run_sql(
         """
-        INSERT INTO read (user_id, site, chapter_id) VALUES (?, ?, ?)
-        ON CONFLICT (user_id, site, chapter_id) DO UPDATE SET updated_at=datetime('now')
+        INSERT INTO read (user_id, site, title_id, chapter_id) VALUES (?,?,?,?)
+        ON CONFLICT (user_id, site, title_id, chapter_id)
+        DO UPDATE SET updated_at=datetime('now');
         """,
-        (user_id, site, chapter_id),
+        (user_id, site, title_id, chapter_id),
     )
 
 
-def unread(user_id, site, chapter_id):
+def unread(user_id, site, title_id, chapter_id):
     run_sql(
-        "DELETE FROM read WHERE user_id=? AND site=? AND chapter_id=?;",
-        (user_id, site, chapter_id),
+        "DELETE FROM read WHERE user_id=? AND site=? AND title_id=? AND chapter_id=?;",
+        (user_id, site, title_id, chapter_id),
     )
 
 
@@ -282,3 +283,32 @@ def find_outdated_titles(since="-6 hours"):
     return run_sql(
         "SELECT id, site FROM title WHERE updated_at <= datetime('now', ?);", (since,)
     )
+
+
+class KeyvalStore:
+    @staticmethod
+    def get(key: str, default=None, since=None) -> str:
+        if since is None:
+            result = run_sql("SELECT value FROM keyval_store WHERE key=?;", (key,))
+        else:
+            result = run_sql(
+                """
+                SELECT value FROM keyval_store WHERE key=?
+                AND updated_at >= datetime('now', ?);
+                """,
+                (key, since),
+            )
+        return result[0] if result else default
+
+    @staticmethod
+    def set(key: str, value: str):
+        # let's not allow crap in by accident
+        assert isinstance(key, str)
+        assert isinstance(value, str)
+        run_sql(
+            """
+            INSERT INTO keyval_store (key, value) VALUES (?,?)
+            ON CONFLICT (key) DO UPDATE SET value=excluded.value, updated_at=datetime('now');
+            """,
+            (key, value),
+        )
